local addon = LibStub("AceAddon-3.0"):NewAddon("Battleground", "AceEvent-3.0", "LibSink-2.0")
addon:SetDefaultModuleState(false)

addon.landmarks = BG_LANDMARKS
local landmarks = BG_LANDMARKS

-- evt funcs
local CheckZone
local AnalyzeLandmarks
local AnalyzeTimeElapsed
local Message

-- random locals
local pois = {}
local oldzone = nil

-- database local ref
local db

local defaults = {
	profile = {
		sink20OutputSink = "RaidWarning",
	}
}

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BattlegroundDB", defaults, "Default")
	self:SetSinkStorage(self.db.profile)
	db = self.db.profile
end

function addon:OnEnable()
	self:RegisterMessage("Battleground_Message", Message)
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", CheckZone)
	CheckZone()
end

function AnalyzeLandmarks()
	for i = 1, GetNumMapLandmarks(), 1 do
		local name, desc, ti, x, y = GetMapLandmarkInfo(i)
		local old = pois[name]
		if landmarks[ti] and ( not old or old ~= ti ) then
			pois[name] = ti
			-- print("BG_P", name, unpack(landmarks[ti]))
			addon:SendMessage("Battleground_POIUpdate", name, ti)
		end
	end
end

function AnalyzeTimeElapsed(offset)
	local start = floor(GetBattlefieldInstanceRunTime()/1000)
	if start < offset then
		addon:SendMessage("Battleground_StartTimer", offset - start)
	else
		addon:SendMessage("Battleground_Started", start - offset)
	end
end

function ChatForward(event, msg)
	addon:SendMessage("Battleground_Message", msg, ChatTypeInfo[strsub(event, 10)])
end

function CheckZone()
	if oldzone then
		wipe(pois)
		for name, module in addon:IterateModules() do module:Disable() end
		oldzone = nil
		addon:UnregisterEvent("WORLD_MAP_UPDATE")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE")
		addon:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE")
	end
	SetMapToCurrentZone()
	local zone = GetMapInfo()
	local module = addon:GetModule(zone, true)
	if module then
		oldzone = zone
		module:Enable()
		addon:RegisterEvent("WORLD_MAP_UPDATE", AnalyzeLandmarks)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL", ChatForward)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE", ChatForward)
		addon:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE", ChatForward)
		AnalyzeLandmarks()
		AnalyzeTimeElapsed(module.startTimer or 122) -- default to 2 minutes
	end
end

function Message(message, text, color, chatFrame)
	if not text then return end
	local r, g, b = 1, 1, 1
	if type(color) == "string" then
		if color == FACTION_ALLIANCE then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		elseif color == FACTION_HORDE then color = ChatTypeInfo["BG_SYSTEM_HORDE"] end
	end
	if type(color) == "table" and color.r and color.g and color.b then
		r, g, b = color.r, color.g, color.b
	end
	addon:Pour(text, r, g, b)
	if chatFrame then
		chatFrame:AddMessage(text, r, g, b)
	end
end

-- GUI drek
local function getButton(frame, orientation)
	local button = CreateFrame("Button", nil, frame)
	button:Show()
	local text = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local time = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
	local icon = button:CreateTexture(nil, "BACKGROUND")
	button:SetWidth(16)
	button:SetHeight(16)
	text:SetWidth(200)
	text:SetHeight(16)
	time:SetWidth(200)
	time:SetHeight(16)
	button.text = text
	button.time = time
	button.icon = icon
	icon:SetPoint("CENTER", button, "CENTER")
	icon:SetWidth(16)
	icon:SetHeight(16)
	icon:SetTexture("Interface\\Minimap\\POIIcons")
	button:SetHeight(16)
	if orientation == "right" then
		time:SetPoint("RIGHT", button, "LEFT")
		text:SetPoint("LEFT", button, "RIGHT")
		time:SetJustifyH("RIGHT")
		text:SetJustifyH("LEFT")
	else
		text:SetPoint("RIGHT", button, "LEFT")
		time:SetPoint("LEFT", button, "RIGHT")
		text:SetJustifyH("RIGHT")
		time:SetJustifyH("LEFT")
	end
	return button
end
addon.getButton = getButton

